pub type Lansky<T> = Result<T, LanskyError>;

#[derive(Debug)]
pub enum LanskyError {
    DuplicateDevice { name: String },
    UnknownDevice { name: String },

    DuplicateInput { name: String, i0: usize, i1: usize },
    DuplicateOutput { name: String, i0: usize, i1: usize },
    
    MissingInput { name: String },
    UnknownInput { name: String },

    WrongNInputs { expected: usize, got: usize },
    WrongNOutputs { expected: usize, got: usize },

    NoFirstBlock, 

    WrongNStack { why: LanskyStackReason, expected: usize, got: usize },

    MissingRegA { i: usize },
    MissingRegZ { i: usize },

    WrongNChannels { expected: usize, got: usize },

    IO(std::io::Error),
}

#[derive(Debug)]
pub enum LanskyStackReason {
    CannotDupEmptyStack,
    CannotOupEmptyStack,
    CannotPopEmptyStack,

    BlockMustEndWithEmptyStack,

    InvokeNInputs
}