// TODOs:
// Generate at 88200Hz, downsample to 44100Hz
// Parser for assembly language
// More context in error messages
// Replace the sample type with something more Pico 8 fixedpoint-like (right now we are always falling down on the i64 interface)
// Use the _top_ part when coercing the sample type to an i32
// jmp and similar instructions
// Use primarily the named interface and allow defaulting
// Channel-like objects that produce a constant value 

use std::{path::Path, fs::File};

use audio::{Sample, Channel};
use device::Device;
use error::{Lansky, LanskyError};
use rack::Rack;
use vm::{Block, BodyInstruction, ExitInstruction, MovSrc, OupDst, RegZ};

mod audio;
mod error;
mod device;
mod standard_devices;
mod rack;
mod vm;

fn main() {
    main_err().expect("crash:");
}


fn main_err() -> Lansky<()> {
    let mut rack = Rack::new();

    rack.add_device(Device::hosted(
        "main".to_string(),
        vec![],
        vec!["audio".to_string()],
        vec![
            Block {
                name: "entry".to_string(),
                body: vec![
                    BodyInstruction::New(MovSrc::Imm(Sample(0)), MovSrc::Imm(Sample(100000)), MovSrc::Imm(Sample(440))),
                    BodyInstruction::Invoke("gensin".to_string(), 1, 1),
                    BodyInstruction::Pop(OupDst::Z(RegZ(0))),
                ],
                exit: ExitInstruction::Exit,
            }
        ],
    )?)?;

    rack.analyze_all()?;

    let result = 
        rack.invoke_positional("main", vec![])?;

    export(Path::new("example.wav"), result)?;

    Ok(())
}

fn export(path: &Path, audio: Vec<Channel>) -> Lansky<()> {
    match &audio[..] {
        [mono] => {
            let header = wav::Header::new(
                wav::header::WAV_FORMAT_PCM,
                1,
                44100,
                16
            );

            let mut out_file = File::create(path).map_err(|e| LanskyError::IO(e))?;
            let data = channel_to_data(&mono);
            wav::write(header, &data, &mut out_file).map_err(|e| LanskyError::IO(e))?;

            Ok(())
        },
        _ => {
            return Err(LanskyError::WrongNChannels { expected: 1, got: audio.len() })
        }
    }
}

fn channel_to_data(channel: &Channel) -> wav::BitDepth {
    let mut v: Vec<i16> = vec![];
    for i in channel.bounds().informal {
        v.push(channel[i].to_i16_sample())
    }
    wav::BitDepth::Sixteen(v)
}

/*
fn main_err() -> std::io::Result<()> {
    let mut out_file = File::create(Path::new("output.wav"))?;
    wav::write(header, &data, &mut out_file)?;

    Ok(())
}
*/