use std::f32::consts::PI;

use crate::{rack::Rack, device::Device, audio::{Channel, Sample}, error::Lansky};

pub fn augment(r: &mut Rack) -> Lansky<()> {
    r.add_device(Device::native(
        "gensin".to_string(), 
        vec!["frequency".to_string()],
        vec!["signal".to_string()],
        gensin
    )?)?;

    Ok(())
}

fn gensin(mut chans: Vec<Channel>) -> Lansky<Vec<Channel>> {
    assert!(chans.len() == 1);
    let inp = chans.pop().expect("must be one channel");

    let mut pos = 0.0;
    let sr = 44100.0;

    let mut oup = Channel::new(inp.formal());

    for i in inp.bounds().informal {
        let freq = inp[i].0 as f32;
        pos += freq/sr;
        pos %= 1.0;
        oup[i] = Sample(((pos * PI * 2.0).sin() * i32::MAX as f32) as i64);
    }

    Ok(vec![oup])
}